# vim: sw=4:ts=4:et

#
# library for Falcon Sandbox
# Author: John Davison
# Originally written for https://github.com/ace-ecosystem/ACE
#

import io
import os
import logging
import json

try:
    from falconpy import FalconXSandbox
except ImportError as no_falconpy:
    raise SystemExit(
        "CrowdStrike FalconPy must be installed in order to use this application."
        ) from no_falconpy

# environment IDs
FS_ENV_LINUX_UBUNTU_16_64BIT = 300
FS_ENV_ANDROID = 200
FS_ENV_WIN_10_64BIT = 160
FS_ENV_WIN_7_64BIT = 110
FS_ENV_WIN_7_32BIT = 100

# sample submission status
FS_STATUS_UNKNOWN = 'unknown'
FS_STATUS_CREATED = 'created'
FS_STATUS_RUNNING = 'running'
FS_STATUS_ERROR = 'error'
FS_STATUS_SUCCESS = 'success'

# action scripts
FS_ACTION_SCRIPT_DEFAULT = 'default'
FS_ACTION_SCRIPT_MAX_ANTIEVASTION = 'default_maxantievasion'
FS_ACTION_SCRIPT_RANDOM_FILES = 'default_randomfiles'
FS_ACTION_SCRIPT_RANDOM_THEME = 'default_randomtheme'
FS_ACTION_SCRIPT_OPEN_IE = 'default_openie'

logger = logging.getLogger('falcon_sandbox')

class FalconSandbox(FalconXSandbox):

    def get_reports(self, debug, target_path, download, summary, *args, **kwargs):
        if summary:
            res = super().get_summary_reports(*args, parameters=None, **kwargs)
            target_path = target_path.format(type='summary.json') if '{type}' in target_path else target_path
        else:
            res = super().get_reports(*args, parameters=None, **kwargs)    
            target_path = target_path.format(type='json') if '{type}' in target_path else target_path

        assert res['body']['resources'], f"Could not get any report. Check your submission ID: \n {res}"
        res = json.dumps(res, indent=4) if debug else json.dumps(res['body']['resources'], indent=4)

        if download:
            with open(target_path, 'w') as f:
                f.write(res)
            logger.info(f"Wrote {target_path}")
        else:
            print(res)

    def get_submissions(self, debug, submitting=False, *args, **kwargs):
        res = super().get_submissions(*args, parameters=None, **kwargs)
        assert res['body']['resources'], f"Could not get submission's status. Check response: \n {res}"

        if submitting:
            return res['body']['resources'][0]['state']

        res = json.dumps(res, indent=4) if debug else json.dumps(res['body']['resources'], indent=4)
        print(res)

    def get_artifact(self, artifact_type, target_path, artifact_id=None, *args, **kwargs):
        res = super().get_reports(*args, parameters=None, **kwargs)
        assert res['body']['resources'], f"Could not get any report. Check your submission ID: \n {res}"

        if artifact_type == 'pcap':
            artifact_id = res['body']['resources'][0]['sandbox'][0].get('pcap_report_artifact_id')

        elif artifact_type == 'memdump':
            artifact_id = res['body']['resources'][0]['sandbox'][0].get('memory_strings_artifact_id')
        
        elif artifact_type == 'iocs.strict.csv':
            artifact_id = res['body']['resources'][0].get('ioc_report_strict_csv_artifact_id')

        elif artifact_type == 'iocs.broad.csv':
            artifact_id = res['body']['resources'][0].get('ioc_report_broad_csv_artifact_id')

        if not artifact_id:
            logger.error(f'{artifact_type} artifact is not available for this report')
            return

        artifact = super().get_artifacts(id=artifact_id)
        target_path = target_path.format(type=artifact_type) if '{type}' in target_path else target_path

        if isinstance(artifact, bytes):
            logger.info(f"Found artifact id {artifact_id} for {artifact_type}")
            with open(target_path, 'wb') as fp:
                f = io.BytesIO(artifact)
                while True:
                    chunk = f.read(io.DEFAULT_BUFFER_SIZE)
                    if not chunk:
                        break
                    fp.write(chunk)
            logger.info(f"Wrote {target_path}")
        else:
            loggger.error(f'Cannot get the file. Check response: {artifact}')

    def get_screenshots(self, target_dir, screenshot_ids=None, *args, **kwargs):
        res = super().get_reports(*args, parameters=None, **kwargs)
        assert res['body']['resources'], f"Could not get any report. Check your submission ID  \n {res}"

        screenshot_ids = res['body']['resources'][0]['sandbox'][0].get('screenshots_artifact_ids')
        
        if not screenshot_ids:
            print(f'Screenshots are not available for this report.')
            return

        os.makedirs(target_dir, exist_ok=True)
        for i, ss_id in enumerate(screenshot_ids):
            screenshot = super().get_artifacts(id=ss_id)
            if isinstance(screenshot, bytes):
                filepath = os.path.join(target_dir, f"screenshot{i + 1}.png")
                with open(filepath, 'wb') as fp:
                    fp.write(screenshot)
                logger.info(f"Wrote {filepath}")
            else:
                logger.error(f'Cannot get screenshot. Check response: \n {screenshot}')

    def get_sample(self, target_path, *args, **kwargs):

        if kwargs['password_protected']:
            target_path = os.path.split(target_path)[0] + 'sample.zip'

        res = super().get_sample(*args, parameters=None, **kwargs)
        if isinstance(res, bytes):
            logger.info(f"Found sample. Attempting to write it...")
            with open(target_path, 'wb') as fp:
                f = io.BytesIO(res)
                while True:
                    chunk = f.read(io.DEFAULT_BUFFER_SIZE)
                    if not chunk:
                        break
                    fp.write(chunk)
            logger.info(f"Wrote {target_path}")
        else:
            logger.error(f'Cannot get the sample. Check response: {res}')

    def delete_report(self, *args, **kwargs):
        res = super().delete_report(*args, parameters=None, **kwargs)
        if not res['body']['errors']:
            logger.info(f"Report {kwargs['ids']} was sucessfully deleted")
        else:
            raise Exception(f"Could not get delete the report. Check response: \n {res}")

    def delete_sample(self, *args, **kwargs):
        res = super().delete_sample(*args, parameters=None, **kwargs)
        if not res['body']['errors']:
            logger.info(f"Sample {kwargs['ids']} was sucessfully deleted")
        else:
            raise Exception(f"Could not get delete the sample. Check response: \n {res}")

    def submit(self, environment_id, url=None, file=None, *args, **kwargs):
        data = {'environment_id': int(environment_id), 
                'url': url}
        if file:
            logger.info("Uploading file...")
            file_name = os.path.split(file)[1]
            data['submit_name'] = file_name
            with open(file, 'rb') as fp:
                file_data = fp.read()
                res = super().upload_sample(file_data=file_data, file_name=file_name)
            assert res['body']['resources'], f"Could not retrieve the file's SHA256. Check response  \n {res}"
            data['sha256'] = res['body']['resources'][0]['sha256']
            logger.info(f"File uploaded sucessfully. Got the file's SHA256 and its name \n {res['body']['resources']}")
        
        supported_arguments = set([
            'action_script',
            'command_line',
            'document_password',
            'network_setting',
            'send_email_notifications',
            'submit_date',
            'submit_time',
            'user_tags'])

        for name, value in kwargs.items():
            if name not in supported_arguments:
                raise KeyError(f"unsupported argument {name}")
            
            if value is not None:
                data[name] = value
        res = super().submit(**data)
        submission_id = res['body']['resources'][0].get('id')
        if not submission_id:
            raise KeyError(f"Could not get submission_id. Check response \n {res}")
        logger.info(f"Got submission id {submission_id} for your submission")
        return submission_id


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""


# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

__version__ = "1.1.0"
description = "Python client library and command line tool for the [Falcon Sandbox API](https://www.falcon-sandbox.com/docs/api/v2)."

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='falcon-sandbox',

    version=__version__,

    description=description,
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/ace-ecosystem/falcon-sandbox',

    author='John Davison & Sean McFeely',
    author_email='mcfeelynaes@gmail.com',

    license='Apache-2.0',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        "Intended Audience :: Information Technology",
        "Intended Audience :: Telecommunications Industry",
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='Information Security,Malware Analysis',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['requests', 'coloredlogs'],
    entry_points={
        'console_scripts': ['falcon-sandbox=falcon_sandbox.cli:main'],
    }
)



#!/usr/bin/env python3

import os
import sys
import time
import json
import base64
import logging
import argparse
import coloredlogs
import pprint

from falcon_sandbox.helpers import load_config, create_user_config
from falcon_sandbox import FS_STATUS_CREATED, FS_STATUS_RUNNING, FalconSandbox

# configure logging #
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s - %(name)s - [%(levelname)s] %(message)s')

logger = logging.getLogger('falcon_sandbox')
coloredlogs.install(level='INFO', logger=logger)

def main():
    parser = argparse.ArgumentParser(description="A command line client for interacting with the Falcon Sandbox library written for the ACE Ecosystem.")
    parser.add_argument('-d', '--debug', action="store_true", help="Set logging to DEBUG", default=False)
    parser.add_argument('--ignore-proxy', action='store_true', default=False, help='Ignore system proxy')
    parser.add_argument('--client-id', action="store", help="Pass client id to use. Overrides config")
    parser.add_argument('--client-secret', action='store',help="Pass client secret to use. Overrides any configured client secrect")

    subparsers = parser.add_subparsers(dest='instruction')

    parser_submit = subparsers.add_parser('submit', help='Upload and submit a sample')
    parser_submit.add_argument('-f', '--file', action='store', help="file to submit")
    parser_submit.add_argument('-u', '--url', action='store', help="url to submit")
    parser_submit.add_argument('-e', '--environment_id', action='store', default='100', help='Enviroment to submit to. Default:100. Use "falcon_sandbox get system -e" to see availble environments')
    parser_submit.add_argument('-o', '--output-target-path', help="Where to write the result file (absolute path) (default: {submission_id}.falcon.{report-or-file_type})", default='{submission_id}.falcon.{type}')
    parser_submit.add_argument('--arguments', action='store', default={}, type=json.loads, help="Arguments to customize submission. Must be a dictionary")

    parser_get = subparsers.add_parser('get', help='Get samples, artifacts, and results from the server')
    parser_get_subs = parser_get.add_subparsers(dest='get_commands')
    
    parser_get_report = parser_get_subs.add_parser('report', help='Get report info')
    parser_get_report.add_argument('submission_id', nargs='+', help="The Falcon submission id")
    parser_get_report.add_argument('-o', '--output-target-path', help="Where to write the result file (absolute path) (default: {submission_id}.falcon.{report-or-file_type})", default='{submission_id}.falcon.{type}')
    parser_get_report.add_argument('-d', '--download', action='store_true', help="Download report")
    parser_get_report.add_argument('-st', '--state', action='store_true', help="Get the submission id report state")
    parser_get_report.add_argument('-sum', '--summary', action='store_true', help="Get the report summary")
    parser_get_report.add_argument('-i', '--iocs', action='store', choices=['strict', 'broad'], help="Get report IOCs")
    parser_get_report.add_argument('-p', '--pcap', action='store_true', help="Get any available pcap for this submission")
    parser_get_report.add_argument('-m', '--memory-dumps', action='store_true', help="Get any available memory dumps for this submission")
    parser_get_report.add_argument('-ss', '--screenshots', action='store_true', help="Get the submission screen shots")
    
    parser_get_sample = parser_get_subs.add_parser('sample', help='Retrieves the file associated with the given ID (SHA256)')
    parser_get_sample.add_argument('file_SHA256', help="the file SHA256")
    parser_get_sample.add_argument('-safe', '--password-protected', action='store_true', help="Zip the sample with password 'infected'")
    parser_get_sample.add_argument('-o', '--output-target-path', help="Where to write the sample file (absolute path) (default: {file_SHA256})", default='{file_SHA256}')

    parser_delete = subparsers.add_parser('delete', help="Delete reports or samples")
    parser_delete.add_argument('-r', '--report', action='store', help="Delete a report based on the submission ID")
    parser_delete.add_argument('-s', '--sample', action='store', help="Delete a sample based on the file SHA256")

    args = parser.parse_args()

    if args.debug:
        coloredlogs.install(level='DEBUG', logger=logger)

    config = load_config(required_options=['ignore_proxy', 'client_id', 'client_secret'])
    if config is None:
        set_config = input("Did not find user configuration, would you like to create one? [Y/n]") or 'Y'
        if set_config == 'Y' or set_config == 'y':
            client_id = input("Client ID of your API client: ")
            client_secret = input("Client secret associated with the client ID: ")
            ignore_proxy = input("Do you need to use the system proxy to connect to the sandbox? [Y/n] ") or 'Y'
            ignore_proxy = True if ignore_proxy.upper() == 'Y' else False
            create_user_config(client_id, client_secret, ignore_proxy)
            config = load_config(required_options=['ignore_proxy', 'client_id', 'client_secret'])
        else:
            return

    if config.getboolean('ignore_proxy') or args.ignore_proxy:
        if 'https_proxy' in os.environ:
            del os.environ['https_proxy']
    
    client_id = args.client_id if args.client_id else config['client_id']
    client_secret = args.client_secret if args.client_secret else config['client_secret']

    falcon = FalconSandbox(client_id=client_id, client_secret=client_secret)

    # if we have a submission_id after running the users commands the 
    # default behavior will be to wait for that job to complete
    # and write the results of the job to a local json file
    submission_id = None

    if args.instruction == 'submit':
        if args.file:
            submission_id = falcon.submit(args.environment_id, file=args.file, **args.arguments)
        if args.url:
            submission_id = falcon.submit(args.environment_id, url=args.url, **args.arguments)

    if args.instruction == 'get':
        if args.get_commands == 'report':
            if args.output_target_path == '{submission_id}.falcon.{type}':
                args.output_target_path = args.output_target_path.format(submission_id=args.submission_id, type='{type}').replace('[', '').replace(']', '').replace("'", '')
            if any((args.state, args.pcap, args.memory_dumps, args.iocs, args.screenshots)):            
                if args.state:
                    falcon.get_submissions(args.debug, ids=args.submission_id)
                if args.pcap:
                    falcon.get_artifact('pcap', args.output_target_path, ids=args.submission_id)
                if args.memory_dumps:
                    falcon.get_artifact('memdump', args.output_target_path, ids=args.submission_id)
                if args.iocs:
                    falcon.get_artifact(f'iocs.{args.iocs}.csv', args.output_target_path, ids=args.submission_id)
                if args.screenshots:
                    target_dir = f"{args.submission_id}.screenshots".replace('[', '').replace(']', '').replace("'", '') if '{type}' in args.output_target_path else args.output_target_path
                    falcon.get_screenshots(target_dir, ids=args.submission_id)
            else:
                falcon.get_reports(args.debug, args.output_target_path, args.download, args.summary, ids=args.submission_id)
        
        if args.get_commands == 'sample':
            if args.output_target_path == '{file_SHA256}':
                args.output_target_path = args.output_target_path.format(file_SHA256=args.file_SHA256)
            falcon.get_sample(args.output_target_path, ids=args.file_SHA256, password_protected=args.password_protected)

    if args.instruction == 'delete':
        if args.report:
            falcon.delete_report(ids=args.report)
        if args.sample:
            falcon.delete_sample(ids=args.sample)

    if submission_id:
        time.sleep(1)
        submission_state = falcon.get_submissions(args.debug, submitting=True, ids=submission_id)
        initial_time = int(time.time())
        while submission_state == FS_STATUS_CREATED or submission_state == FS_STATUS_RUNNING:
            if (int(time.time()) - initial_time) % 10 == 0:
                initial_time += 9 # essentially, log again about every 9 seconds
                logger.info(f"Submission {submission_id} is in {submission_state.upper()} state...")
                submission_state = falcon.get_submissions(args.debug, submitting=True, ids=submission_id)
        logger.info(f"Submission {submission_id} has moved to {submission_state.upper()} state")
        if args.output_target_path == '{submission_id}.falcon.{type}':
            args.output_target_path = args.output_target_path.format(submission_id=args.submission_id, type='{type}')
        falcon.get_reports(args.debug, args.output_target_path, True, False, ids=submission_id)